indexing
	description: "Evolution of a species - a sequence of generations, wich consist of individuals, represented by their genetic code"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	EVOLUTION
inherit
	COUNTABLE_SEQUENCE[POPULATION]
	rename
		item as current_population
	redefine
		has, index_of, occurrences, search
	end
	STORABLE
create
	make

feature -- Initialization
	make(p: INDIVIDUAL) is
			-- Set all evolution parameters to defaults and `prototype' to `p'
		require
			p_exist: p /= Void
		do
			prototype := p
			population_size := default_population_size
			survivers_part := default_survivers_part
			survivers_competition := default_survivers_competition
			parents_competition := default_parents_competition
			index := 1
		ensure
			prototype_set: prototype = p
			population_size_default: population_size = default_population_size
			survivers_part_default: survivers_part = default_survivers_part
			survivers_competition_default: survivers_competition = default_survivers_competition
			parents_competition_default: parents_competition = default_parents_competition
			index_equals_one: index = 1
		end

feature -- Access
	population_size: INTEGER
			-- Population size

	survivers_part: REAL
			-- Relative quantity of individuals from old population that move to a new one

	survivers_competition: INTEGER
			-- How many individuals compete for surviving in next generation?

	parents_competition: INTEGER
			-- How many individuals compete for becoming a parent?

	i_th (i: INTEGER_32): POPULATION is
			-- Population of i-th generation
		local
			count: INTEGER
		do
			if i > 1 and then i >= last_item then
				-- No need to start from beginning
				Result := last_result
				count := last_item
			else
				-- Start from scratch
				create Result.make(1, population_size)
				Result.randomize(prototype)
				Result.evaluate
				count := 1
			end
			from
			until
				count = i
			loop
				Result := next_generation(Result)
				count := count + 1
			end
			last_result := Result
			last_item := i
		ensure then
			size_correct: Result.count = population_size
		end

	best: INDIVIDUAL is
			-- The best individual from `current_population'
		do
			Result := current_population.best
		end

	inds: ARRAY [INDIVIDUAL] is
		local
			p: POPULATION
			i: INTEGER
		do
			p := current_population
			create Result.make (p.lower, p.upper)
			from i := p.lower
			until i > p.upper
			loop
				Result.put (p.item (i), i)
				i := i + 1
			end
		end

feature -- Basic operations
	set_population_size (s: INTEGER) is
			-- Set `population_size' to `s'
		require
			s_positive: s > 0
		do
			population_size := s
		ensure
			population_size_set: population_size = s
		end

	set_survivers_part (p: REAL) is
			-- Set `survivers_part' to `p'
		require
			p_not_negative: p >= 0.0
			p_not_too_large: p <= 1.0
		do
			survivers_part := p
		ensure
			survivers_part_set: survivers_part = p
		end

	set_survivers_competition (c: INTEGER) is
			-- Set `survivers_competition' to `c'
		require
			c_not_too_small: c >= 1
		do
			survivers_competition := c
		ensure
			survivers_competition_set: survivers_competition = c
		end

	set_parents_competition (c: INTEGER) is
			-- Set `parents_competition' to `c'
		require
			c_not_too_small: c >= 1
		do
			parents_competition := c
		ensure
			parents_competition_set: parents_competition = c
		end

feature {NONE} -- Implementation
	last_item: INTEGER
			-- Last `item' requested

	last_result: POPULATION
			-- Value from last call to `item'

	prototype: INDIVIDUAL
			-- The Great Ancestor

	next_generation (p: POPULATION) : POPULATION is
			-- Form next generation population from `p'
			-- Main feature of evolution, can be redefined in descendants
			-- to alternate genetic algorithm strategies
		require
			p_exists: p /= Void
		local
			temp: POPULATION
		do
			create Result.make(p.lower, p.upper)
				-- Some individuals survive
			if surviver_count > 0 then
				temp := p.selected(surviver_count, survivers_competition)
				Result.subcopy(temp, temp.lower, temp.upper, Result.lower)
			end
				-- The rest of `Result' is filled with children, possibly mutated
			if surviver_count < Result.count then
				temp := p.children(Result.count - surviver_count, parents_competition)
				Result.subcopy(temp.mutants, temp.lower, temp.upper, Result.lower + surviver_count)
			end
			Result.evaluate
		ensure
			result_exists: Result /= Void
		end

	surviver_count: INTEGER is
			-- Absolute number of survivers
		do
			Result := (survivers_part*population_size).rounded
		end

	default_population_size: INTEGER is 100
	default_survivers_part: REAL is 0.1
	default_survivers_competition: INTEGER is 4
	default_parents_competition: INTEGER is 4

feature {NONE} -- Inapplicable
	has (v: like current_population): BOOLEAN is
			-- May become applicable if evolution history is stored
		do
		end

	index_of (v: like current_population; i: INTEGER_32): INTEGER_32 is
			-- May become applicable if evolution history is stored
		do
		end

	occurrences (v: like current_population): INTEGER_32 is
			-- May become applicable if evolution history is stored
		do
		end

	search (v: like current_population) is
			-- May become applicable if evolution history is stored
		do
		end

invariant
	prototype_exists: prototype /= Void
	population_size_positive: population_size > 0
	survivers_part_not_negative: survivers_part >= 0.0
	survivers_part_not_too_large: survivers_part <= 1.0
	surviver_count_not_negative: surviver_count >= 0
	surviver_count_not_too_large: surviver_count <= population_size
	survivers_competition_not_too_small: survivers_competition >= 1
	parents_competition_not_too_small: parents_competition >= 1
end
